# -*- coding: utf-8 -*-
from django.contrib import admin

from django import forms
from django.conf import settings
from django.utils.html import format_html
from django.utils.translation import ugettext_lazy
from django.utils.six import with_metaclass
try:
    from django.urls import reverse
except ImportError:  # django < 2
    from django.core.urlresolvers import reverse


from job_offers.models import Application, JobOffer, RelatedObject

try:
    from djangocms_text_ckeditor.fields import HTMLFormField
except ImportError:
    HTMLFormField = None


@admin.register(RelatedObject)
class RelatedObjectAdmin(admin.ModelAdmin):
    """Contract type for Job offers"""
    list_display = [
        'name',
        'type',
        'image_preview',
        'slug',
        'weight',
        'is_active',
    ]
    list_filter = [
        'type',
    ]
    readonly_fields = [
    ]

    def image_preview(self, instance):
        if not instance or not instance.image:
            return ''
        return format_html('<img class="relatedobject-image-preview"'
                           ' src="{}" style="max-width: 3em;" title="" />', instance.image.url, instance)

    def get_readonly_fields(self, request, instance=None):
        ro_fields = list(super(RelatedObjectAdmin, self).get_readonly_fields(request, instance))
        if instance is not None:
            ro_fields.append('type')
        return ro_fields


class RelatedObjectFields(type(forms.ModelForm)):
    def __new__(self, name, bases, attrs):
        required_fields = set(getattr(settings, 'JOB_OFFER_RELATED_OBJECTS_REQUIRED', None) or [])

        for slug, related_name in RelatedObject._meta.get_field('type').choices:
            attrs[slug] = forms.ModelChoiceField(
                label=related_name,
                queryset=RelatedObject.objects.filter(type=slug),
                required=slug in required_fields,
            )
        return super(RelatedObjectFields, self).__new__(self, name, bases, attrs)


class JobOfferForm(with_metaclass(RelatedObjectFields, forms.ModelForm)):
    if HTMLFormField:
        competences = JobOffer._meta.get_field('competences').formfield(
            widget=None,
            form_class=HTMLFormField,
        )
        short_description = JobOffer._meta.get_field('short_description').formfield(
            widget=None,
            form_class=HTMLFormField,
        )
        job_description = JobOffer._meta.get_field('job_description').formfield(
            widget=None,
            form_class=HTMLFormField,
        )

    def __init__(self, *args, **kw):
        if kw.get('instance'):
            initial = dict(kw['instance']._relations)
            initial.update(kw.get('initial') or [])
            kw['initial'] = initial
        super(JobOfferForm, self).__init__(*args, **kw)

    class Meta:
        model = JobOffer
        fields = [
            'title',
            'slug',
            'short_description',
            'job_description',
            'competences',
            'reference',
            'date',
            'expire',
            'weight',
            'is_active',
            'tags',
        ]

    def save(self, **kw):
        instance = super(JobOfferForm, self).save(**kw)

        def save_relations():
            for slug, related_name in RelatedObject._meta.get_field('type').choices:
                setattr(instance, slug, self.cleaned_data[slug])

        try:
            super_save_m2m = self.save_m2m

            def save_m2m():
                super_save_m2m()
                save_relations()
            self.save_m2m = save_m2m
        except AttributeError:
            save_relations()

        return instance


def build_admin_related_object(slug):
    def build_admin_related_object_inner(self, instance):
        return getattr(instance, slug)
    build_admin_related_object_inner.short_description = slug
    return build_admin_related_object_inner


class RelatedObjectFieldsAdmin(type(admin.ModelAdmin)):
    def __new__(self, name, bases, attrs):
        fields = []
        for slug, related_name in RelatedObject._meta.get_field('type').choices:
            fields.append(slug)
            attrs[slug] = build_admin_related_object(slug)

        cls = super(RelatedObjectFieldsAdmin, self).__new__(self, name, bases, attrs)
        cls.list_display.extend(fields)
        return cls


@admin.register(JobOffer)
class JobOfferAdmin(with_metaclass(RelatedObjectFieldsAdmin, admin.ModelAdmin)):
    """Job offers"""
    form = JobOfferForm

    search_fields = [
        'title',
        'reference',
    ]
    list_filter = [
        'date',
        'expire',
    ]
    list_display = [
        'title',
        'reference',
        'date',
        'weight',
    ]
    prepopulated_fields = {
        'slug': ['title']
    }

    def view_on_site(self, instance):
        return reverse('job-offer-detail', args=[instance.slug])


class ApplicationModelForm(forms.ModelForm):
    """Forms for Application"""
    message = forms.CharField(
        widget=forms.Textarea
    )

    class Meta:
        model = Application
        fields = [
            'job_offer',
            'message',
            'last_name',
            'first_name',
            'email',
            'phone',
            'cv',
            'cover_letter',
            'other_document',
        ]


@admin.register(Application)
class ApplicationAdmin(admin.ModelAdmin):
    """Job application"""
    form = ApplicationModelForm

    search_fields = [
        'last_name',
        'email',
        'phone',
    ]
    list_filter = [
        'date',
    ]
    list_display = [
        'display',
        'date',
        'last_name',
        'email',
        'job_location',
        'job_area',
        'get_cv',
    ]

    def display(self, application):
        if application.job_offer is None:
            return ugettext_lazy(u'Speculative application')
        return str(application.job_offer)

    def job_area(self, obj):
        if obj.job_offer is None:
            return ''
        return obj.job_offer.geo_area

    def job_location(self, obj):
        if obj.job_offer is None:
            return ''
        return obj.job_offer.location

    def get_cv(self, obj):
        if obj.cv:
            return format_html(u'<a href="{}">{} {}</a>', obj.cv.url, obj.first_name, obj.last_name)
        else:
            return ugettext_lazy(u'No CV')

    get_cv.short_description = u'Cv'
