# -*- coding: utf-8 -*-

import time
from email import utils
import lxml.etree as ET

from django.conf import settings
from django.views.generic.base import View
from django.views.generic.list import MultipleObjectMixin
from django.contrib.sites.shortcuts import get_current_site
from django.utils.html import strip_tags
from django.utils.six import text_type, PY2
from django.utils.module_loading import import_string
from django.http import HttpResponse
try:
    from django.urls import reverse
except ImportError:  # django < 2
    from django.core.urlresolvers import reverse

from job_offers.models import JobOffer


def format_date(date):
    nowtimestamp = time.mktime((date.year, date.month, date.day, 0, 0, 0, 0, 0, 0))
    return utils.formatdate(nowtimestamp)


class XMLJobSerializer(object):
    def __init__(self, job, site):
        self.job = job
        self.site = site

    def _build_url(self, job):
        return 'https://{}{}'.format(self.site.domain, reverse('job-offer-detail', args=[self.job.slug]))

    def get_title(self):
        return self.job.title

    def get_referencenumber(self):
        return self.job.reference

    def get_date(self):
        return format_date(self.job.date)

    def get_url(self):
        return self._build_url(self.job)

    def get_description(self):
        return strip_tags(self.job.job_description)

    def get_country(self):
        return self.job.country

    def has_geo_area(self):
        return self.job.geo_area

    def get_geo_area(self):
        return self.job.geo_area

    def has_location(self):
        return self.job.location

    def get_location(self):
        return self.job.location

    def has_contract_type(self):
        return self.job.contract_type

    def get_contract_type(self):
        return self.job.contract_type

    def has_categories(self):
        return self.job.category or self.job.occupation or self.job.sector

    def get_categories(self):
        return u', '.join(text_type(category)
                          for category in
                          [self.job.category, self.job.occupation, self.job.sector]
                          if category)

    def __iter__(self):
        yield 'title', self.get_title()
        yield 'date', self.get_date()
        yield 'referencenumber', self.get_referencenumber()
        yield 'url', self.get_url()
        yield 'description', self.get_description()
        yield 'country', self.get_country()
        if self.has_geo_area():
            yield 'state', self.get_geo_area()
        if self.has_location():
            yield 'city', self.get_location()
        if self.has_contract_type():
            yield 'jobtype', self.get_contract_type()
        if self.has_categories():
            yield 'categories', self.get_categories()


class XMLFeed(object):
    def __init__(self, site, objects, generation=None):
        self.site = site
        self.objects = objects
        self.generation = generation or time.time()

    def _element_cdata(self, tag, text):
        el = ET.Element(tag)
        el.text = ET.CDATA(text_type(text))
        return el

    def _element(self, tag, text):
        el = ET.Element(tag)
        el.text = text_type(text)
        return el

    def to_element(self):
        source = ET.Element('source')
        source.append(self._element('publisher', self.site.name))
        source.append(self._element('publisherurl', 'https://{}'.format(self.site.domain)))
        source.append(self._element('lastBuildDate', utils.formatdate(self.generation)))
        for job in self.objects:
            source.append(self._encode_job(job))
        return source

    def __bytes__(self):
        return ET.tostring(self.to_element(), encoding='UTF-8')
    if PY2:
        __str__ = __bytes__

    @property
    def content_type(self):
        return 'application/xml; charset=utf-8'

    def get_serializer_class(self):
        serializer_class_name = getattr(settings, 'JOB_OFFER_JOB_SERIALIZER', None)
        if not serializer_class_name:
            return XMLJobSerializer
        return import_string(serializer_class_name)

    def get_serializer(self, job):
        serializer_class = self.get_serializer_class()
        return serializer_class(job, self.site)

    def _encode_job(self, job):
        tag = ET.Element('job')
        serializer = self.get_serializer(job)
        for tag_name, text in serializer:
            tag.append(self._element_cdata(tag_name, text))
        return tag


class IndeedXMLFeedView(MultipleObjectMixin, View):
    queryset = JobOffer.objects.active()

    def get(self, request, *args, **kw):
        objects = self.get_queryset()
        site = get_current_site(self.request)
        feed = XMLFeed(site, objects)
        return HttpResponse(
            bytes(feed),
            content_type=feed.content_type
        )
